<?php

namespace WebsiteBundle\Controller\Site;

use Bazookas\CommonBundle\Exception\NotImplementedException;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use WebsiteBundle\Controller\Base\BaseUserController;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\Participant;
use WebsiteBundle\Entity\Quote;
use WebsiteBundle\Entity\TestSession;
use WebsiteBundle\Entity\UnlockedQuote;

class UserController extends BaseUserController
{

  public function homeAction(Request $request)
  {
    try {
      $templateData = $this->getBasicData();
    } catch (AccessDeniedException $e) {
      return $this->handleAccessDeniedException($request, 'user_login');
    }

    $sessionRepo = $this->get('doctrine.orm.entity_manager')->getRepository(TestSession::class);
    $templateData['lastSession'] = $sessionRepo->getLastSession($this->getUser(), $templateData['experiment']);

    return $this->render('@Website/User/home.html.twig', $templateData);
  }

  public function viewStatisticsAction(Request $request)
  {
    try {
      $templateData = $this->getBasicData();
    } catch (AccessDeniedException $e) {
      return $this->handleAccessDeniedException($request, 'user_login');
    }

    $participant = $this->getUser();

    $userStatisticsService = $this->get('WebsiteBundle.Services.UserStatisticsService');
    $stats = $userStatisticsService->getStatisticsData($participant);

    $entityRepo = $this->get('doctrine.orm.entity_manager')->getRepository(TestSession::class);
    $lastSession = $entityRepo->getLastSession($participant, $templateData['experiment']);

    list($unViewedCount, $quotes) = $this->getUnlockedQuotes($templateData['experiment'], $participant);

    $templateData['unViewedCount'] = $unViewedCount;
    $templateData['quotes'] = $quotes;
    $templateData['stats'] = $stats;
    $templateData['lastSession'] = $lastSession;

    return $this->render('@Website/User/statistics.html.twig', $templateData);
  }

  private function getUnlockedQuotes(Experiment $experiment, Participant $participant)
  {
    $participantRepo = $this->get('doctrine.orm.entity_manager')->getRepository(Participant::class);

    $unlockedQuotes = $participantRepo->getUnlockedQuotes($participant);
    $availableQuotes = $experiment->getAvailableQuotes()->toArray();

    $unViewedCount = 0;
    foreach ($availableQuotes as $key => $availableQuote) {
      foreach ($unlockedQuotes as $unlockedQuote) {
        if ($unlockedQuote->getQuote()->getId() === $availableQuote->getId()) {
          $availableQuotes[$key] = $unlockedQuote;
          if (!$unlockedQuote->isViewed()) {
            $unViewedCount++;
          }

          break;
        }
      }
    }

    return [$unViewedCount, $availableQuotes];
  }

  /**
   * @param UnlockedQuote|Quote $quoteA
   * @param UnlockedQuote|Quote $quoteB
   * @return int
   */
  private function quoteCompareEquals($quoteA, $quoteB)
  {
    if ($quoteA instanceof UnlockedQuote) {
      $idOfA = $quoteA->getQuote()->getId();
      $idOfB = $quoteB->getId();
    } else if ($quoteB instanceof UnlockedQuote) {
      $idOfA = $quoteA->getId();
      $idOfB = $quoteB->getQuote()->getId();
    } else {
      $idOfA = $quoteA->getId();
      $idOfB = $quoteB->getId();
    }

    if ($idOfA > $idOfB) {
      return 1;
    } else if ($idOfA < $idOfB) {
      return -1;
    } else {
      return 0;
    }
  }

}
