<?php

namespace WebsiteBundle\Form\Admin;

use Bazookas\AdminBundle\Form\Type\NestedFormType;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\UrlType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Translation\TranslatorInterface;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\ExperimentCondition;
use WebsiteBundle\Entity\Test;

class ExperimentConditionAdminType extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $experiment = $options['experiment'];
    $experimentIsActive = !empty($experiment) ? $experiment->getSessionCount() > 0 : false;

    $builder
      ->add('testGroup', ChoiceType::class, [
        'label' => 'admin.entities.experimentCondition.fields.testGroup',
        'choices' => [
          'admin.entities.test.enums.TYPE_APASAT' => Test::TYPE_APASAT,
          'admin.entities.test.enums.TYPE_CONTROL' => Test::TYPE_CONTROL
        ],
        'disabled' => $experimentIsActive
      ])
      ->add('gamificationLevel', ChoiceType::class, [
        'label' => 'admin.entities.experimentCondition.fields.gamificationLevel',
        'choices' => [
          'admin.entities.experimentCondition.enums.GAMIFICATION_LOW' => ExperimentCondition::GAMIFICATION_LOW,
          'admin.entities.experimentCondition.enums.GAMIFICATION_HIGH' => ExperimentCondition::GAMIFICATION_HIGH
        ],
        'expanded' => true,
        'multiple' => false,
        'disabled' => $experimentIsActive
      ])
      ->add('startSurveyUrl', UrlType::class, [
        'label' => 'admin.entities.experimentCondition.fields.startSurveyUrl',
        'disabled' => $experimentIsActive,
        'tooltip_title' => 'admin.entities.experimentCondition.tooltips.survey.title',
        'tooltip_content' => $options['surveyUrlToolTipBody'],
        'tooltip_type' => 'popover',
        'tooltip_trigger' => 'click'
      ])
      ->add('endSurveyUrl', UrlType::class, [
        'label' => 'admin.entities.experimentCondition.fields.endSurveyUrl',
        'disabled' => $experimentIsActive,
        'tooltip_title' => 'admin.entities.experimentCondition.tooltips.survey.title',
        'tooltip_content' => $options['surveyUrlToolTipBody'],
        'tooltip_type' => 'popover',
        'tooltip_trigger' => 'click'
      ])
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => ExperimentCondition::class,
      'translation_domain' => 'admin',
      'experiment' => null,
      'surveyUrlToolTipBody' => ''
    ]);
  }

}
