<?php

namespace WebsiteBundle\Form\Site;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\CallbackTransformer;
use Symfony\Component\Form\Extension\Core\Type\DateTimeType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\GreaterThan;
use Symfony\Component\Validator\Constraints\LessThan;

class ReminderType extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('nextSession', DateTimeType::class, [
        'constraints' => [
          new GreaterThan($options['minDateTime']->format(\DateTime::ATOM)),
          new LessThan($options['maxDateTime']->format(\DateTime::ATOM))
        ]
      ])
    ;

    $builder->get('nextSession')
      ->addModelTransformer(new CallbackTransformer(
        function ($datetime) {
          //transforms the original value into a format that'll be used to render the field
          return $datetime;
        },
        function ($datetime) {
          //transforms the submitted value back into the format you'll use in your code

          //The date is sent as a belgian datetime but symfony assumes it's in UTC time so re-make the object
          $submitted = new \DateTime($datetime->format('Y-m-d H:i:s'), new \DateTimeZone('Europe/Brussels'));

          $submitted->setTimeZone(new \DateTimeZone('UTC'));

          return $submitted;
        }
      ))
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'minDateTime' => null,
      'maxDateTime' => null,
      'csrf_protection' => false
    ]);
  }
}
