var bazookas = bazookas || {};

bazookas.graphs = (function () {
  var self = {};
  var accuracyCtx = $("#accuracyChard");
  var speedctx = $("#speedChard");
  var scoreCtx = $("#scoreChard");
  var $settings = $('#settings');

  self.init = function (context) {

    if($('#stats').length > 0){
      setTabs();
      createGraphs();
      setAchievementUnlockEvents();
    }

    $('.js-show-quote').on('click', quoteShowClickHandler);
    $('.modal__close').on('click', closeQuoteModalClickHandler);
  };

  function setAchievementUnlockEvents(){
    $('.grid__block').on('click', achievementClickedHandler);
  }

  function closeQuoteModalClickHandler(){
    $('#achievementModal').toggleClass("show");
  }

  function quoteShowClickHandler(e){
    var $quote = $(e.currentTarget);
    var $sessionNumber = $('#sessionNumber');
    var $sessionQuote = $('#sessionQuote');
    var $sessionDate = $('#sessionDate');

    $sessionNumber.text($quote.find('.session__number').text());
    $sessionQuote.text($quote.data('quote'));
    $sessionDate.text($quote.data('quote-date'));

    var $achievementModal = $('#achievementModal');
    if($quote.hasClass('unlocked')){
      $achievementModal.addClass('unlocked').removeClass('master');
    }

    if($quote.hasClass('master')){
      $achievementModal.addClass('master').removeClass('unlocked');
    }

    $achievementModal.toggleClass("show");
  }

  function achievementClickedHandler(e) {
    var $block = $(e.currentTarget);
    if($block.hasClass('new')){
      unlockNewAchievement($block);
    }
  }

  function unlockNewAchievement(achievement){
    //Mark the element as viewed
    $.ajax({
      url: achievement.data('url'),
      type: "POST",
      data: JSON.stringify({
        unlockedQuoteId: achievement.data('unlocked-id')
      }),
      contentType: "application/json; charset=utf-8",
      dataType: "json",
      success: function(response) {
        //do nothing
      },
      error: function(response) {
        console.error(response);
      }
    });

    achievement.removeClass('new');
    achievement.addClass('unlocked');

    achievement.find('.fa').remove();
    achievement.find('p').removeClass('hide');
  }

  function setTabs(){
    $('#tabs li a:not(:first)').addClass('inactive');
    $('.container').hide();
    $('.container:first').show();

    $('#tabs li a').on('click', function(){
      var t = $(this).attr('href');
      if($(this).hasClass('inactive')){ //added to not animate when active
        $('#tabs li a').addClass('inactive');
        $(this).removeClass('inactive');
        $('.container').hide();
        $(t).fadeIn('slow');
      }
      return false;
    });
  };

  function createAccuracyGraph() {
    var data = $settings.data('accuracy');
    var xAxisLabels = [];
    var counter = 1;

    if (data.length > 0) {
      data.forEach(function(item){
        counter++;
        xAxisLabels.push('Sessie ' + counter);
      });

      new Chart(accuracyCtx, {
        type: 'line',
        data: {
          labels: xAxisLabels,
          datasets: [{
            label: 'Aantal antwoorden correct (in %)',
            fill: false,
            lineTension: 0.1,
            backgroundColor: "rgba(75,192,192,0.4)",
            borderColor: "rgba(75,192,192,1)",
            borderCapStyle: 'butt',
            borderDash: [],
            borderDashOffset: 0.0,
            borderJoinStyle: 'miter',
            pointBorderColor: "rgba(75,192,192,1)",
            pointBorderWidth: 1,
            pointHoverRadius: 5,
            pointHoverBackgroundColor: "rgba(75,192,192,1)",
            pointHoverBorderColor: "rgba(220,220,220,1)",
            pointHoverBorderWidth: 2,
            pointRadius: 3,
            pointBackgroundColor: "rgba(75,192,192,1)",
            pointHitRadius: 10,
            data: data,
            spanGaps: false,
          }]
        }, options: {
          responsive: true,
          maintainAspectRatio: true,
          scales: {
            yAxes: [
              {
                scaleLabel: {
                  display: true,
                  labelString: 'Procent'
                }
              }
            ]
          }
        }
      });
    }
  }

  function createSpeedGraph() {
    var data = $settings.data('speed');
    var legend = [];
    var counter = 1;

    if (data.length > 0) {
      data.forEach(function (item) {
        counter++;
        legend.push('Sessie ' + counter);
      });

      new Chart(speedctx, {
        type: 'line',
        data: {
          labels: legend,
          datasets: [{
            label: 'Snelheid van de opeenvolging van getallen (in seconden)',
            fill: false,
            lineTension: 0.1,
            backgroundColor: "rgba(75,192,192,0.4)",
            borderColor: "rgba(75,192,192,1)",
            borderCapStyle: 'butt',
            borderDash: [],
            borderDashOffset: 0.0,
            borderJoinStyle: 'miter',
            pointBorderColor: "rgba(75,192,192,1)",
            pointBorderWidth: 1,
            pointHoverRadius: 5,
            pointHoverBackgroundColor: "rgba(75,192,192,1)",
            pointHoverBorderColor: "rgba(220,220,220,1)",
            pointHoverBorderWidth: 2,
            pointRadius: 3,
            pointBackgroundColor: "rgba(75,192,192,1)",
            pointHitRadius: 10,
            data: data,
            spanGaps: false,
          }]
        }, options: {
          responsive: true,
          maintainAspectRatio: true,
          scales: {
            yAxes: [
              {
                scaleLabel: {
                  display: true,
                  labelString: 'Seconden'
                }
              }
            ]
          }
        }
      });
    }
  }

  function createScoreGraph() {
    if (scoreCtx) {
      var data = $settings.data('score');
      var legend = [];
      var counter = 1;

      if (data && data.length > 0) {
        data.forEach(function (item) {
          counter++;
          legend.push('Sessie ' + counter);
        });

        new Chart(scoreCtx, {
          type: 'line',
          data: {
            labels: legend,
            datasets: [{
              label: 'Score (in punten)',
              fill: false,
              lineTension: 0.1,
              backgroundColor: "rgba(75,192,192,0.4)",
              borderColor: "rgba(75,192,192,1)",
              borderCapStyle: 'butt',
              borderDash: [],
              borderDashOffset: 0.0,
              borderJoinStyle: 'miter',
              pointBorderColor: "rgba(75,192,192,1)",
              pointBorderWidth: 1,
              pointHoverRadius: 5,
              pointHoverBackgroundColor: "rgba(75,192,192,1)",
              pointHoverBorderColor: "rgba(220,220,220,1)",
              pointHoverBorderWidth: 2,
              pointRadius: 3,
              pointBackgroundColor: "rgba(75,192,192,1)",
              pointHitRadius: 10,
              data: data,
              spanGaps: false,
            }]
          }, options: {
            responsive: true,
            maintainAspectRatio: true,
            scales: {
              yAxes: [
                {
                  scaleLabel: {
                    display: true,
                    labelString: 'Score'
                  }
                }
              ]
            }
          }
        });
      }
    }
  }

  function createGraphs() {
    createAccuracyGraph();
    createSpeedGraph();
    createScoreGraph();
  }

  return self;
})();

