<?php
namespace Bazookas\AdminBundle\Form\Type;

use Bazookas\AdminBundle\Util\FilterField;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;

class FilterType extends AbstractType {

  private $translator;

  public function __construct($translator) {
    $this->translator = $translator;
  }

  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('field', ChoiceType::class, array(
        'label' => false,
        'choices' => $this->createFieldChoiceOptions($builder->getData()),
        'choice_attr' => $this->createFieldChoiceAttributes($builder->getData()),
        'attr' => array(
          'class' => 'js-filter-field'
        )
      ))
      ->add('operator', ChoiceType::class, array(
        'label' => false,
        'attr' => array(
          'class' => 'js-filter-operator',
          'data-initially-selected' => $builder->getData()->getOperator()
        ),
        'choices' => FilterField::getDefaultOperators()
      ))
      ->add('value', TextType::class, array(
        'label' => false,
        'attr' => array(
          'class' => 'js-filter-value'
        ),
        'required' => false,
        'empty_data' => false
      ))
      ->add('submit', SubmitType::class, array(
        'label' => 'admin.filter.buttons.filter',
        'attr' => array(
          'class' => 'js-filter-submit btn btn-primary'
        )
      ))
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'data_class' => 'Bazookas\AdminBundle\Entity\Filter',
      'csrf_protection' => false,
      'translation_domain' => 'admin'
    ));
  }

  private function createFieldChoiceOptions($filter) {
    $choices = [];

    dump($filter);
    die;

    foreach($filter->getFilterFields() as $field) {
      $choices[$field->label] = $field->field;
    }

    return $choices;
  }

  //see: http://symfony.com/blog/new-in-symfony-2-7-choice-form-type-refactorization
  private function createFieldChoiceAttributes($filter) {
    $attributes = [];

    foreach($filter->getFilterFields() as $field) {
      //prepare the operators
      $operators = [];
      foreach($field->getOperators() as $key => $operator) {
        $key = $this->translator->trans($key, [], 'admin');

        $operators[$key] = $operator;
      }

      $attributes[$field->label] = array(
        'data-operators' => json_encode($operators),
        'data-type' => $field->getType()
      );
    }

    return $attributes;
  }
}
