## Form Grouping

You can group form fields in order to render each group as a seperate panel. Let's assume you have the following form class:

```php
<?php

namespace WebsiteBundle\Form\Admin;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolver;
use WebsiteBundle\Entity\Experiment;

class ExperimentAdminType extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $experiment = $builder->getData();
    $experimentIsActive = !empty($experiment) ? $experiment->getSessionCount() > 0 : false;

    $builder
      //Panel 1
      ->add('name', TextType::class, [
        'label' => 'admin.entities.experiment.fields.name'
      ])

      //Panel 2
      ->add('totalSessions', TextType::class, [
        'label' => 'admin.entities.experiment.fields.totalSessions'
      ])
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => Experiment::class,
      'renderType' => 'panel',  //TODO future version, allow the form to be rendered in different ways
      'grouping' => [
        'admin.entities.experiment.forms.panelHeader1' => [
          'name',
        ],
        'admin.entities.experiment.forms.panelHeader2' => [
          'totalSessions',
        ],
      ],
    ]);
  }

  public function buildView(FormView $view, FormInterface $form, array $options)
  {
    if (isset($options['grouping'])) {
      $view->vars['grouping'] = $options['grouping'];
    }
  }

}
```

In order to render the fields `name` and `totalSessions` in their own panels, expand the form's options by setting the default `grouping` to the field grouping. The `grouping` property is an associative array of the format: `<panel header translation key> => ['field1', 'field2']`.
