## Getting started

This documentation will attempt to provide a bit more information on how to set up oAuth2 authentication for API endpoints.

### installing dependencies

This bundle requires the [FOSAuthServerBundle](https://github.com/FriendsOfSymfony/FOSOAuthServerBundle) to work. For installation instructions please visit their github. NOTE: if you are starting from the symfony starter repo then you don't need to install the dependy, its has already been added for you.

### Configuration - config.yml

In config.yml add the following configuration.

```YAML
fos_oauth_server:
  db_driver: orm
  client_class:        APIBundle\Entity\OAuth2Client
  access_token_class:  APIBundle\Entity\OAuth2AccessToken
  refresh_token_class: APIBundle\Entity\OAuth2RefreshToken
  auth_code_class:     APIBundle\Entity\OAuth2AuthCode
  service:
    user_provider: APIFrameworkBundle.user_provider.BaseUserProvider
    options:
      # Changing tokens and authcode lifetime
      access_token_lifetime: 86400     # 1 day
      refresh_token_lifetime: 1209600  # 14 days
```

As you can see in the config above the client, access token, refresh token and auth code classes can be customised to fit the project's particular needs. A base implementation of these classes can be found here:

- [Bazookas/APIFrameworkBundle/Entity/Entity/OAuth2AccessToken](Entity/OAuth2AccessToken.php)
- [Bazookas/APIFrameworkBundle/Entity/Entity/OAuth2AuthCode](Entity/OAuth2AuthCode.php)
- [Bazookas/APIFrameworkBundle/Entity/Entity/OAuth2Client](Entity/OAuth2Client.php)
- [Bazookas/APIFrameworkBundle/Entity/Entity/OAuth2RefreshToken](Entity/OAuth2RefreshToken.php)

In most project specific cases you will need to extend these to add extra properties to the access token and the refresh tokens (e.g. a link to a user entity.)

### Configuration - security.yml

In security.yml you define the firewalls and the access controle checks that need to occur. A sample configuration is shown below: 

```YAML
firewalls:
    insecure:
        pattern:  ^\/(media|cron|(api[\/v[\d+\.\d+]*]*\/doc)|(api[\/v[\d+\.\d+]*]*\/(%requiredlocales%)+/forgotPassword))
        security: false
    create_user:
        pattern: ^\/api[\/v[\d+\.\d+]*]*\/(%requiredlocales%)+/user
        methods: [ PUT ]
        security: false
    oauth_token:
        pattern:    ^/api/v1.0/(%requiredlocales%)+/token
        security:   false
    api:
        pattern:    ^/api
        fos_oauth:  true
        stateless:  true
        provider: base_user_provider
```

Firewalls are checked from top to bottom; this means that firewalls defined first will be matched first if they meet the desired criteria. As shown above an `insecure` firewall is added for all routes which do not require authentication this is marked by the `security: false` property. The `pattern` field accepts regex, if the url matches the given regex the firewall authentication will be invoked. 

To enable OAuth2 authentication for a firewall use the `fos_oauth:  true` configuration, the `stateless` key indicates whether or not a session should be used with the api, in almost all case this will always be `stateless: true` (true = don't use a session)

The `create_user` firewall provides an example of limiting authentication based on the HTTP method this allows the same url to be used to create, read, update, delete a user as needed.

Some routes always need to be insecure, for example the `oauth_token` route needs to be made insecure because otherwise we cannot request a security token without providing a security token.

### Creating new OAuth Clients

create new OAuth clients by using the command below:

`bin/console oauth-server:client:create --grant-type=password --grant-type=refresh_token`

available grant types are:

```
    password => user/password authentication
    refresh_token => authentication by refresh_token
    client_credentions => app level authentication (when you wish to perform non-user specific calls)
    http://facebook.com => facebook authentication
```
For more information on the facebook grant-type:
- [Facebook grant type](facebook.md)
